@@ -1,12 +1,13 @@
 """A Vulnerability Scanner that combines results from all configured scanners"""
 # pylint: disable=duplicate-code
+import traceback
 from pathlib import Path
 from typing import List
 
 import typer
-from security_commons.common.reporting.models import ReportFormat
-from security_commons.common.reporting.reporting import Reporting
-from security_commons.common.utils import parse_sbom, parse_sbom_json_string
+from hoppr_security_commons.reporting.models import ReportFormat
+from hoppr_security_commons.reporting import Reporting
+from hoppr_security_commons.utils import parse_sbom, parse_sbom_json_string
 from typer import Typer
 
 from hopprcop.combined.combined_scanner import CombinedScanner
@@ -27,7 +28,7 @@ def vulnerability_report(
         help="The report formats to generate",
     ),
     output_dir: Path = typer.Option(
-        Path.cwd(), help="The directory where reports will be writen"
+        Path.cwd(), help="The directory where reports will be written"
     ),
     base_report_name: str = typer.Option(
         None, help="The base name supplied for the generated reports"
@@ -41,8 +42,12 @@ def vulnerability_report(
         ),
         envvar="OS_DISTRIBUTION",
     ),
+    trace: bool = typer.Option(
+        False, help="Print traceback information on unhandled error"
+    ),
 ):
     """Generates vulnerability reports based on the specified BOM and formats"""
+
     try:
         if base_report_name is None:
             if bom.endswith(".json"):
@@ -55,14 +60,20 @@ def vulnerability_report(
         reporting = Reporting(output_dir, base_report_name)
         combined = CombinedScanner()
         grype_scanner = GrypeScanner()
+        trivy_scanner = TrivyScanner()
         grype_scanner.grype_os_distro = os_distro
+        trivy_scanner.trivy_os_distro = os_distro
         combined.set_scanners(
-            [grype_scanner, TrivyScanner(), OSSIndexScanner(), GemnasiumScanner()]
+            [grype_scanner, trivy_scanner, OSSIndexScanner(), GemnasiumScanner()]
         )
 
         parsed_bom = None
 
         if bom.endswith(".json") or bom.endswith(".xml"):
+            if not Path(bom).exists():
+                msg = typer.style(f"{bom} does not exist", fg=typer.colors.RED)
+                typer.echo(msg)
+                raise typer.Exit(code=1)
             parsed_bom = parse_sbom(Path(bom))
         else:
             parsed_bom = parse_sbom_json_string(bom, "The json provided sbom")
@@ -70,4 +81,10 @@ def vulnerability_report(
         results = combined.get_vulnerabilities_by_sbom(parsed_bom)
         reporting.generate_vulnerability_reports(formats, results, parsed_bom)
     except Exception as exc:  # pylint: disable=broad-except
-        print(f"unexpected error: {exc}")
+        if trace:
+            print(traceback.format_exc())
+        msg = typer.style(
+            f"unexpected error: {exc}",
+            fg=typer.colors.RED,
+        )
+        typer.echo(msg)
