@@ -23,29 +23,33 @@ import subprocess
 import tempfile
 import time
 import zipfile
+
 from pathlib import Path
 from typing import Optional
 from urllib.parse import urlparse
+
+import hoppr_cyclonedx_models.cyclonedx_1_4 as cdx
 import requests
 import typer
 import yaml
-from cvss import CVSS2, CVSS3
-from hoppr_cyclonedx_models.cyclonedx_1_4 import Vulnerability, Rating, Severity, Tool
-from packageurl import PackageURL
 
-from security_commons.common.utils import (
-    get_vulnerability_source,
+from cvss import CVSS2, CVSS3
+from packageurl import PackageURL
+from hoppr_security_commons.utils import (
     get_advisories_from_urls,
     get_references_from_ids,
+    get_vulnerability_source,
 )
-from security_commons.common.vulnerability_scanner import VulnerabilitySuper
+from hoppr_security_commons.vulnerability_scanner import VulnerabilitySuper
+
 from hopprcop.gemnasium.models import GemnasiumVulnerability
 
 
 class GemnasiumScanner(VulnerabilitySuper):
     """A Vulnerability Scanner for Gitlab's Gemnasiumm Database"""
 
-    supported_formats = ["npm", "maven", "pypi", "gem", "golang", "connan"]
+    supported_formats = ["npm", "maven", "pypi", "gem", "golang", "conan"]
+    # TODO: supported_formats is not used; should get_vulnerabilities_by_purl be filtering purls by it?
 
     database_path = None
     # url = "https://gitlab.com/gitlab-org/security-products/gemnasium-db/-/archive/master/gemnasium-db-master.zip"
@@ -69,12 +73,13 @@ class GemnasiumScanner(VulnerabilitySuper):
         """If the ruby semver command isn't installed then extract from this package"""
         data = pkgutil.get_data(__name__, "semver").decode("utf-8")
         self.semver_path = Path(tempfile.gettempdir()) / "semver"
-        with open(self.semver_path, "w", encoding="UTF-8") as file:
-            file.write(data)
-            file.close()
-        os.chmod(
-            self.semver_path,
-            stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH,
+        self.semver_path.write_text(data, encoding="utf-8")
+        self.semver_path.chmod(
+            mode=stat.S_IRWXU
+            | stat.S_IRGRP
+            | stat.S_IXGRP
+            | stat.S_IROTH
+            | stat.S_IXOTH
         )
 
     def __download_and_extract_database(self):
@@ -134,38 +139,38 @@ class GemnasiumScanner(VulnerabilitySuper):
 
     def get_vulnerabilities_by_purl(
         self, purls: list[PackageURL]
-    ) -> dict[str, Optional[list[Vulnerability]]]:
+    ) -> dict[str, Optional[list[cdx.Vulnerability]]]:
         """Get the vulnerabilities for a list of package URLS (purls)
         This function will return a dictionary of package URL to vulnerabilities or none if no vulnerabilities are found
         """
         vulnerabilities_by_purl = {}
         for purl in purls:
-            vulnerabilities_by_purl[purl.to_string()] = []
-            path = self.__get_path(purl)
-            if path.exists():
-                for filename in os.listdir(path):
-                    try:
-                        if (path / filename).is_file():
-                            with open(path / filename, "r", encoding="UTF-8") as file:
-                                data = yaml.full_load(file)
-                                file.close()
-                                vuln = GemnasiumVulnerability(**data)
+            purl_str = purl.to_string()
+            vulnerabilities_by_purl[purl_str] = []
 
-                                if self.__is_affected_range(
-                                    purl.type, purl.version, vuln.affected_range
-                                ):
-                                    vulnerability = self.__convert_to_cyclone_dx(vuln)
-                                    if len(vulnerability.ratings) > 0:
-                                        vulnerabilities_by_purl[
-                                            purl.to_string()
-                                        ].append(vulnerability)
-                    except:  # pylint: disable=bare-except
-                        print(f"failed to parse gemnasium file for {purl}")
+            if not (path := self.__get_path(purl)).exists():
+                continue
+
+            vuln_files = [file for file in path.glob("*") if file.is_file()]
+
+            for vuln_file in vuln_files:
+                try:
+                    data = yaml.full_load(vuln_file.read_text(encoding="utf-8"))
+                    vuln = GemnasiumVulnerability(**data)
+
+                    if self.__is_affected_range(
+                        purl.type, purl.version, vuln.affected_range
+                    ):
+                        vulnerability = self.__convert_to_cyclone_dx(vuln)
+                        if len(vulnerability.ratings) > 0:
+                            vulnerabilities_by_purl[purl_str].append(vulnerability)
+                except:  # pylint: disable=bare-except
+                    print(f"failed to parse gemnasium file for {purl}")
 
         return vulnerabilities_by_purl
 
     @staticmethod
-    def __convert_to_cyclone_dx(vuln: GemnasiumVulnerability) -> Vulnerability:
+    def __convert_to_cyclone_dx(vuln: GemnasiumVulnerability) -> cdx.Vulnerability:
         # pylint: disable=duplicate-code
         """Converts a gemnasium vulnerabity to a vulnerability"""
         res = list(filter(lambda x: "cve-" in x.lower(), vuln.identifiers))
@@ -173,7 +178,7 @@ class GemnasiumScanner(VulnerabilitySuper):
         cwes = []
         if vuln.cwe_ids is not None:
             cwes = list(map(lambda x: int(x.replace("CWE-", "")), vuln.cwe_ids))
-        cyclone_vuln = Vulnerability(
+        cyclone_vuln = cdx.Vulnerability(
             id=vuln_id,
             recommendation=vuln.solution,
             cwes=cwes,
@@ -188,24 +193,24 @@ class GemnasiumScanner(VulnerabilitySuper):
         if vuln.cvss_v3 is not None:
             cvss = CVSS3(vuln.cvss_v3)
             cyclone_vuln.ratings.append(
-                Rating(
+                cdx.Rating(
                     score=cvss.base_score,
-                    severity=Severity[cvss.severities()[0].lower()],
-                    method="CVSSv3",
+                    severity=cdx.Severity[cvss.severities()[0].lower()],
+                    method=cdx.ScoreMethod.CVSSv3,
                     vector=str(cvss.clean_vector()),
                 )
             )
         elif vuln.cvss_v2 is not None:
             cvss = CVSS2(vuln.cvss_v2)
             cyclone_vuln.ratings.append(
-                Rating(
+                cdx.Rating(
                     score=cvss.base_score,
-                    severity=Severity[cvss.severities()[0].lower()],
-                    method="CVSSv2",
+                    severity=cdx.Severity[cvss.severities()[0].lower()],
+                    method=cdx.ScoreMethod.CVSSv2,
                     vector=cvss.clean_vector(),
                 )
             )
-        cyclone_vuln.tools = [Tool(vendor="Gitlab", name="Gemnasium")]
+        cyclone_vuln.tools = [cdx.Tool(vendor="Gitlab", name="Gemnasium")]
         return cyclone_vuln
 
     @staticmethod
@@ -216,19 +221,16 @@ class GemnasiumScanner(VulnerabilitySuper):
 
         return Path(tempfile.gettempdir())
 
-    def __get_path(self, purl: PackageURL):
+    def __get_path(self, purl: PackageURL) -> Path:
         """build a path to the gemnasium path"""
         repo_format = purl.type
         if repo_format == "npm":
-            if purl.namespace != "" and purl.namespace is not None:
-                path_slug = f"npm/{purl.namespace}/{purl.name}"
-            else:
-                path_slug = f"npm/{purl.name}"
+            path_slug = Path("npm") / (purl.namespace or "") / purl.name
         elif repo_format == "maven":
-            path_slug = f"maven/{purl.namespace}/{purl.name}"
+            path_slug = Path("maven") / purl.namespace / purl.name
         elif repo_format == "golang":
-            path_slug = f"go/{purl.namespace}/{purl.name}"
+            path_slug = Path("go") / purl.namespace / purl.name
         else:
-            path_slug = f"{repo_format}/{purl.name}"
+            path_slug = Path(repo_format) / purl.name
 
-        return self.database_path / Path(path_slug)
+        return self.database_path / path_slug
