@@ -1,24 +1,17 @@
 import uuid
+
 from copy import deepcopy
 from pathlib import Path
-from typing import List
+from typing import Any
 
-from hoppr.base_plugins.hoppr import HopprPlugin, hoppr_process
-from hoppr.result import Result
-from hoppr.hoppr_types.bom_access import BomAccess
-from hoppr_cyclonedx_models.cyclonedx_1_4 import (
-    Component,
-    CyclonedxSoftwareBillOfMaterialsStandard as Bom_1_4,
-    Vulnerability,
-    Affect,
-)
-
-from security_commons.common.reporting.reporting import Reporting
-from security_commons.common.reporting.models import ReportFormat
-from security_commons.common.vulnerability_combiner import combine_vulnerabilities
-from hopprcop.combined.combined_scanner import CombinedScanner
+from hoppr import BomAccess, Component, HopprPlugin, Result, Sbom, hoppr_process
+from hoppr_cyclonedx_models.cyclonedx_1_4 import Affect, Vulnerability
+from hoppr_security_commons.reporting.models import ReportFormat
+from hoppr_security_commons.reporting import Reporting
+from hoppr_security_commons.vulnerability_combiner import combine_vulnerabilities
 
 from hopprcop import __version__
+from hopprcop.combined.combined_scanner import CombinedScanner
 
 
 class HopprCopPlugin(HopprPlugin):
@@ -43,7 +36,6 @@ class HopprCopPlugin(HopprPlugin):
         Supply sbom to hoppr cop to perform vulnerabilty check
         """
         self.get_logger().info("[ Executing hopprcop vulnerability check ]")
-        self.get_logger().flush()
 
         output_dir = self.config.get(
             "output_dir", Path(self.context.collect_root_dir, "generic")
@@ -66,9 +58,9 @@ class HopprCopPlugin(HopprPlugin):
         # Map bom ref to results - uses purl as ref
         bom_ref_to_results = dict[str, self.ComponentVulnerabilityWrapper]()
 
-        # Map purls to components
+        # Map purls to components, try to normalize purls to lowercase
         purl_to_component = {
-            component.purl: component for component in parsed_bom.components
+            component.purl.lower(): component for component in parsed_bom.components
         }
 
         # Delivered Bom version
@@ -88,66 +80,98 @@ class HopprCopPlugin(HopprPlugin):
 
         # Build dictionary to go from bom-ref to vulnerabilities
         for purl in results:
-            component = purl_to_component[purl]
-            bom_ref = component.purl  # component.bom_ref.__root__
+            # Different purl values being returned from different scanners, attempt to normalize the data
+            # by making it lower case and if not found, reversing pypi expectations with - to _.
+            adjusted_purl = purl.lower()
+            if adjusted_purl not in purl_to_component and "pypi" in adjusted_purl:
+                adjusted_purl = adjusted_purl.replace("-", "_")
 
-            if len(parsed_bom.vulnerabilities) > 0:
-                # Account for existing vulnerabilites on bom
-                for existing_vulnerability in parsed_bom.vulnerabilities:
-                    for affect in existing_vulnerability.affects:
-                        if (
-                            affect.ref == bom_ref
-                            or affect.ref == component.bom_ref.__root__
-                        ):
-                            results[purl].append(existing_vulnerability)
+            if adjusted_purl in purl_to_component:
+                component = purl_to_component[adjusted_purl]
+                bom_ref = component.purl  # component.bom_ref
+                if (
+                    len(parsed_bom.vulnerabilities) > 0
+                    and bom_ref not in bom_ref_to_results
+                ):
+                    # Account for existing vulnerabilites on bom
+                    for existing_vulnerability in parsed_bom.vulnerabilities:
+                        for affect in existing_vulnerability.affects:
+                            if affect.ref in {bom_ref, component.bom_ref}:
+                                results[purl].append(existing_vulnerability)
 
-                results[purl] = combine_vulnerabilities([{purl: results[purl]}])[0]
+                    results[purl] = combine_vulnerabilities([{purl: results[purl]}])[0]
 
-            updated_results = deepcopy(results[purl])
+                updated_results = deepcopy(results[purl])
 
-            wrapper = self.ComponentVulnerabilityWrapper(
-                bom_serial_number, bom_version, updated_results
-            )
-            bom_ref_to_results[bom_ref] = wrapper
+                if bom_ref not in bom_ref_to_results:
+                    wrapper = self.ComponentVulnerabilityWrapper(
+                        bom_serial_number, bom_version, updated_results
+                    )
+                    bom_ref_to_results[bom_ref] = wrapper
+                elif (
+                    len(bom_ref_to_results[bom_ref].vulnerabilities) > 0
+                    and len(updated_results) > 0
+                ):
+                    existing_vulnerabilities: list[Vulnerability] = bom_ref_to_results[
+                        bom_ref
+                    ].vulnerabilities
+                    bom_ref_to_results[
+                        bom_ref
+                    ].vulnerabilities = combine_vulnerabilities(
+                        [{purl: updated_results + existing_vulnerabilities}]
+                    )[
+                        0
+                    ]
+            else:
+                self.get_logger().info(f"Could not find purl ({purl}) in component map")
 
         hoppr_delivered_bom = self.__perform_hoppr_bom_updates(
             reporting, deepcopy(parsed_bom), bom_ref_to_results, formats
         )
         self.__perform_hopprcop_reporting(reporting, parsed_bom, results, formats)
 
+        self.get_logger().flush()
+
         return Result.success(return_obj=hoppr_delivered_bom)
 
     def __add_bom_ref_and_flatten(
         self,
         reporting: Reporting,
-        bom_ref_to_component: dict[str, List[Component]],
+        bom_ref_to_component: dict[str, list[Component]],
         external_ref: bool = False,
-    ) -> List[Vulnerability]:
-        flattened_vulnerabilities: List[Vulnerability] = []
+    ) -> list[Vulnerability]:
+        flattened_vulnerabilities: list[Vulnerability] = []
+        # Capture vulnerability id to vulnerability to account multiple components affected by the same vulnerability
+        vuln_id_to_vuln = {}
         for bom_ref in bom_ref_to_component:
             for vuln in bom_ref_to_component[bom_ref].vulnerabilities:
-                vuln.affects = [] if vuln.affects is None else vuln.affects
+                existing_vuln = vuln
+                if existing_vuln.id not in vuln_id_to_vuln:
+                    existing_vuln.affects = []
+                else:
+                    existing_vuln = vuln_id_to_vuln[vuln.id]
+
                 if external_ref:
-                    vuln.affects.append(
+                    existing_vuln.affects.append(
                         Affect(
-                            **{
-                                "ref": f"urn:cdx:{bom_ref_to_component[bom_ref].serial_number}/{bom_ref_to_component[bom_ref].version}#{bom_ref}"
-                            }
+                            ref=f"urn:cdx:{bom_ref_to_component[bom_ref].serial_number}/{bom_ref_to_component[bom_ref].version}#{bom_ref}"
                         )
                     )
                 else:
-                    vuln.affects.append(Affect(**{"ref": bom_ref}))
+                    existing_vuln.affects.append(Affect(ref=bom_ref))
 
-                flattened_vulnerabilities.append(vuln)
+                vuln_id_to_vuln[vuln.id] = existing_vuln
 
+        flattened_vulnerabilities = list(vuln_id_to_vuln.values())
         flattened_vulnerabilities.sort(key=reporting.get_score, reverse=True)
+
         return flattened_vulnerabilities
 
     def __perform_hoppr_bom_updates(
         self,
         reporting: Reporting,
-        parsed_bom: Bom_1_4,
-        bom_ref_to_results: dict[str, any],
+        parsed_bom: Sbom,
+        bom_ref_to_results: dict[str, Any],
         formats,
     ):
         if self.EMBEDDED_VEX in formats:
@@ -157,7 +181,7 @@ class HopprCopPlugin(HopprPlugin):
             # Existing vulnerabilities were accounted for
             parsed_bom.vulnerabilities = []
             Reporting.add_vulnerabilities_to_bom(parsed_bom, flattened_results)
-        elif self.LINKED_VEX in formats and self.EMBEDDED_VEX not in formats:
+        elif self.LINKED_VEX in formats:
             flattened_results = self.__add_bom_ref_and_flatten(
                 reporting, bom_ref_to_results, True
             )
@@ -177,15 +201,13 @@ class HopprCopPlugin(HopprPlugin):
     def __perform_hopprcop_reporting(
         self,
         reporting: Reporting,
-        parsed_bom: Bom_1_4,
-        results: dict[str, List[Vulnerability]],
-        formats: List[str],
+        parsed_bom: Sbom,
+        results: dict[str, list[Vulnerability]],
+        formats: list[str],
     ):
-        filtered_formats = list(
-            filter(lambda x: x != self.LINKED_VEX and x != self.EMBEDDED_VEX, formats)
-        )
-
-        if len(filtered_formats) > 0:
+        if filtered_formats := list(
+            filter(lambda fmt: fmt not in {self.LINKED_VEX, self.EMBEDDED_VEX}, formats)
+        ):
             filtered_formats = [
                 ReportFormat[format.upper()] for format in filtered_formats
             ]
@@ -200,7 +222,7 @@ class HopprCopPlugin(HopprPlugin):
             self.vulnerabilities = vulnerabilities
 
 
-def get_scanners() -> List[str]:
+def get_scanners() -> list[str]:
     """Defines scanners to use for hoppr cop"""
     return [
         "hopprcop.gemnasium.gemnasium_scanner.GemnasiumScanner",
